#appModules/teamspeak.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012-2015 NVDA Contributors <http://www.nvda-project.org/>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

import re
import controlTypes
from NVDAObjects.UIA import UIA, TreeviewItem
import api
import UIAHandler as uiah
import appModuleHandler
import scriptHandler
import ui, speech
import tones

import languageHandler, addonHandler
addonHandler.initTranslation()

class AppModule(appModuleHandler.AppModule):
	def event_NVDAObject_init(self,obj):
		role = obj.role
		# Silence controls that are containers with odd roles.
		if role in [controlTypes.ROLE_SPLITBUTTON]:
			obj.isPresentableFocusAncestor = False
			return
		# Use FullDescription when available for controls without a name.
		self.nameViaFullDescription(obj)

	def event_nameChange(self, obj, nextHandler):
		# Announce changes to the status bar.
		if not isinstance(obj, UIA): return
		if obj.role != controlTypes.Role_StatusBar: return
		ui.message(obj.name)
		nextHandler()

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		role = obj.role
		if role == controlTypes.ROLE_TREEVIEW:
			try: clsList.remove(UIA)
			except ValueError: pass
			clsList.insert(0, TSTree)
		elif role == controlTypes.ROLE_TREEVIEWITEM:
			try: clsList.remove(TreeviewItem)
			except ValueError: pass
			clsList.insert(0, TSTreeItem)

	def nameViaFullDescription(self,obj):
		"""Use the UIA FullDescription property to name otherwise unnamed controls.
		Watch out for HTML though, like for the master volume control when on screen.
		"""
		if obj.name: return
		try: fdesc = obj._getUIACacheablePropertyValue(uiah.UIA_FullDescriptionPropertyId)
		except AttributeError: fdesc = ""
		if obj.role == controlTypes.ROLE_GROUPING and fdesc.startswith("<"):
			# Remove XML content.
			fdesc = re.sub(r'<[^>]+?>', r'', fdesc)
			try: obj.description = ""
			except: pass
		obj.name = fdesc

	def event_eventQueued(self, obj, *args, **kwargs):
		"Custom testing event, not called by native NVDA."
		try: name = obj.name
		except: name = ""
		try: rtext = controlTypes.roleLabels[obj.role]
		except: rtext = ""
		try: evname = kwargs["originalEventName"]
		except: evname = ""
		msg = " ".join([name, rtext, evname])
		if not msg: return
		ui.message(msg)

# A quick flexible class for making structures.
class Struct: pass

class TSTree(UIA):
	def _get_treeInfo(self):
		"""Return a structure of information about this tree:
			baseLevel: The left indent of the outermost node (level 0).
			delta: The screen distance between any two tree levels.
		"""
		info = Struct()
		info.baseLevel = 0
		info.delta = 0
		# This should be the root node.
		obj = self.firstChild
		left0 = obj.location[0]
		info.baseLevel = left0
		# Skip to the next node that indents under this one. By tree design, it will be at the next level in if found.
		while obj and obj.location[0] <= left0:
			obj = obj.next
		# This is the distance between root node and the first level within it.
		self.treeInfo = info
		if obj: info.delta = obj.location[0] -left0
		return info

class TSTreeItem(TreeviewItem):
	def initOverlayClass(self):
		try:
			if self.columnNumber == 1 and self.next.columnNumber != 2:
				self.columnNumber = None
		except AttributeError: pass

	def _get_positionInfo(self):
		# In case TeamSpeak's trees start behaving properly someday.
		pos = super(TSTreeItem, self)._get_positionInfo()
		#if pos.has_key("level"): return pos
		tree = self.parent
		info = tree.treeInfo
		left = self.location[0] -info.baseLevel
		level = None
		if info.delta: level = int(left / info.delta)
		if level is None: return pos
		pos["level"] = level
		return pos

	def _get_states(self):
		states = super(TSTreeItem, self)._get_states()
		if controlTypes.STATE_EXPANDED not in states: return states
		states1 = states.difference(set([controlTypes.STATE_EXPANDED]))
		l1 = self.positionInfo.get("level")
		if l1 is None: return states
		n = self.next
		# If there's nothing after this, there can't be descendants of this node.
		if n is None: return states1
		l2 = n.positionInfo.get("level")
		if l2 is None or l2 > l1: return states
		# Same or lesser level here means no descendants of self.
		return states1

